/*
 - SpUtils.cpp
 -
 - Some utility functions.
 -
 - This file is public domain. Do whatever you want with it.
 -
 - 2001.03.08 Bernard Krummenacher    Created this file.
 -
*/

#include "stdio.h"

#include "SpUtils.h"
#include "SpLocaleApp.h"

int						// Number of bytes written to the BString.
BStringPrintf	// Fill a BString with formatted data.
(
	BString* String,		// The Destination BString object.
	const char* Format,	// The format string.
	...									// The arguments list.
)
{	// This is the most stupid implementation possible.
	char Buffer[4096]; 								// Very big buffer.
	va_list ArgList;

	va_start(ArgList,Format);				 	// Get the first argument.
	vsprintf(Buffer,Format,ArgList); 	// Fill the buffer.
	*String = Buffer;								 	// Copy to BString object.
}

const char* WeekDays[] = {"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"};

const char* FullWeekDays[] = {"Sunday", "Monday", "Tuesday", "Wednesday",
														 "Thursday", "Friday", "Saturday"};

const char* MonthDays[] = {"Jan", "Feb", "Mar", "Apr", "May_[1]", "Jun",
												 "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};

const char* FullMonthDays[] = {"January", "February", "March",
															"April", "May_[2]", "June",
														  "July", "August", "September",
														  "October", "November", "December"};

// Localized version of strftime().
size_t strftime_locale(char *str, size_t maxsize, const char *format, const struct tm *timeptr)
{
	BString Buffer;
	
	for (unsigned int i = 0; i < strlen(format); i++)
	{
		if (format[i] == '%')
		{
			i++;
			switch (format[i])
			{
			case 'a':
				Buffer << SpTranslate(WeekDays[timeptr->tm_wday]);
				break;
			case 'A':
				Buffer << SpTranslate(FullWeekDays[timeptr->tm_wday]);
				break;
			case 'b':
				Buffer << SpTranslate(MonthDays[timeptr->tm_mon]);
				break;
			case 'B':
				Buffer << SpTranslate(FullMonthDays[timeptr->tm_mon]);
				break;
 			case 'c':
				Buffer << "??"; // What is appropriate?
				break;
			case 'd':
				if (timeptr->tm_mday < 10)
				{
					Buffer << '0';
				}
				Buffer << timeptr->tm_mday;
				break;
			case 'H':
				if (timeptr->tm_hour < 10)
				{
					Buffer << '0';
				}
				Buffer << timeptr->tm_hour;
				break;
			case 'I':
				if ((((timeptr->tm_hour + 11) % 12) + 1) < 10)
				{
					Buffer << '0';
				}
				Buffer << (((timeptr->tm_hour + 11) % 12) + 1);
				break;
			case 'j':
				Buffer << timeptr->tm_yday + 1;
				break;
			case 'm':
				if (timeptr->tm_mon < 9)
				{
					Buffer << '0';
				}
				Buffer << timeptr->tm_mon + 1;
				break;
			case 'M':
				if (timeptr->tm_min < 10)
				{
					Buffer << '0';
				}
				Buffer << timeptr->tm_min;
				break;
			case 'p':
				Buffer << ((timeptr->tm_hour / 12)?"PM":"AM");
				break;
			case 'S':
				if (timeptr->tm_sec < 10)
				{
					Buffer << '0';
				}
				Buffer << timeptr->tm_sec;
				break;
			case 'U':
				Buffer << "-??-"; // How to do this?
				break;
			case 'w':
				Buffer << timeptr->tm_wday;
				break;
			case 'W':
				Buffer << "-??-"; // How to do this?
				break;
			case 'x':
				Buffer << "??"; // What is appropriate?
				break;
			case 'X':
				Buffer << "??"; // What is appropriate?
				break;
			case 'y':
				if ((timeptr->tm_year % 100) < 10)
				{
					Buffer << '0';
				}
				Buffer << (timeptr->tm_year % 100);
				break;
			case 'Y':
				Buffer << timeptr->tm_year + 1900;
				break;
			case 'Z':
				Buffer << ""; // Timezone unavailable.
				break;
			default:
				Buffer << '%';
				Buffer << format[i];
			}
		}
		else
		{
			Buffer += format[i];
		}
	}
	strncpy(str,Buffer.String(),maxsize);
	str[maxsize-1] = 0;
}
